%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                                               %
% Übung: Aussagenlogik                                                                          %
% Aussagenlogik kann durch die folgende kontextfreie Grammatik definiert werden:                %
% [Hinweis: Die Operatoren sind mit einem "\" gekennzeichnet, wie LaTeX-Befehle]                %
%                                                                                               %
% prop --> p                                                                                    %
% prop --> q                                                                                    %
% prop --> r                                                                                    %
% prop --> \not prop                  [In LaTeX: \neg prop]                                     %
% prop --> (prop \and prop)           [In LaTeX: \land prop]                                    %
% prop --> (prop \or prop)            [In LaTeX: \lor prop]                                     %
% prop --> (prop \implies prop)       [In LaTeX: \to prop]                                      %
%                                                                                               %
% Schreiben Sie eine DCG, die diese Sprache generiert.                                          %
% Um die Schreibweise für Prolog einfacher verständlich zu machen und die Aussagen zu Listen    %
% umzuformen, werden Sie eine alternative Schreibweise verwenden müssen.                        %
% Zum Beispiel wird die Aussage: \not(p \implies q), zu: [not, '(', p, implies, q, ')']         %
%                                                                                               %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%
%% Erläuterung:
% In den vorherigen Aufgaben haben wir bereits verschiedene Sprachen in DCGs konvertiert und
% uns auch mit der Notation und deren Umwandlung vertraut gemacht. Auch sollte uns klar sein,
% warum und wie wir die Aussagenlogik formal als kontextfreie Sprache definieren können.
% Falls nicht, schaue dir den unten verlinkten Foliensatz an, der das Thema noch einmal vertieft.
% Zu beachten gilt auch, dass die Grammatik nur die wohlgeformtheit der Formel überprüft, aber
% nicht deren Konsistenz, Ausssagekraft oder Funktion.


%
%%% Lösung:

% Zuerst definieren wir die geforderten drei Terminale (p,q,r).
% Dies sollte mittlerweile eine triviale Aufgabe darstellen.
prop --> [p].                                   % p wird als Terminal initialisiert
prop --> [q].                                   % q wird als Terminal initialisiert
prop --> [r].                                   % r wird als Terminal initialisiert

% Anschließend die einstellige Negation, wieder trivial.
prop --> [not], prop.                           % Negation initialisieren

% Hier nun die abstrakte Konstruktion für die zweistelligen Konjunktionen,
% die weiter unten definiert werden. Hier gilt es zu beachten, dass
% die Klammern obligatorisch sind um ein wohlgeformtes Ergebnis zu erhalten.
prop --> ['('],prop, conj, prop,[')'].          

% Die drei Konjunktionen (und, oder, impliziert) als Terminale:
conj --> [and].                                 % and wird als Terminal initialisiert
conj --> [or].                                  % or wird als Terminal initialisiert
conj --> [implies].                             % implies wird als Terminal initialisiert


%
%% Testen (optional)
% Diese Tests kannst du manuell in deinem Prolog Interpreter durchführen.
% [Hinweis: Wenn der äußerste Operator eine Konjunktion ist, dann ist eine äußere Klammer notwendig]

% Peirce's law:
% ?- prop(['(', '(', '(', p, implies, q, ')', implies, p, ')', implies, p, ')'],[]).
% -> true.

% Clavius's law:
% ?- prop(['(', '(', not, p, implies, p, ')', implies, p, ')'],[]).
% -> true.

% Nicht Wohlgeformt:
% ?- prop([not, '(', p, implies, q, implies, q, ')'],[]).
% -> false.


%
%% Weitere Informationen:

% Kontextfreie Grammatiken: http://www.thi.informatik.uni-frankfurt.de/lehre/dismod/ws1415/Kontextfreie%20Grammatiken%20Folien.pdf
% Peirce's law: https://en.wikipedia.org/wiki/Peirce%27s_law
% Clavius's law: https://en.wikipedia.org/wiki/Consequentia_mirabilis
