%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                 %
% Übung: cfg_zu_dcg_2                                             %
% Gib eine kontextfreie Grammatik an, die die Sprache generiert,  %      
% die aus allen Palindromen über dem Alphabet {c, d} besteht.     %
%                                                                 %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%
%% Erläuterung:
% Eine kontextfreie Grammatik ist eine formale Grammatik, bei der jede linke Regelseite aus 
% genau einem Nichtterminalsymbol besteht.
% Kontextfreie Grammatiken lassen sich in Prolog folgendermaßen leicht als DCG darstellen:
% Nichtterminale werden kleingeschrieben und Terminale stehen kleingeschrieben in einer Liste.
% Die Übergänge werden durch einen Pfeil bestehend aus zwei Bindestrichen und dem größer-Zeichen dargestellt: -->
% Eine Grammatikregel der Form C → c sieht in Prolog-Notation also so aus: c --> [c].
% Das leere Wort wird als leere Liste dargestellt.
% 
% Ein Wort ist ein Palindrom, wenn es von vorne und von hinten gelesen gleich ist (siehe Übung "palindrom" aus Woche 6).


%
%%% Lösung:
% Um eine Grammatik zu erstellen, die die Sprache generiert, welche aus allen Palindromen über dem Alphabet {c, d} besteht, 
% müssen wir folgendes sicherstellen:
% Ein Buchstabe, der generiert wird, muss entweder alleine in der Mitte eines Wortes stehen,
% oder im vorderen Teil des Wortes sowie im hinteren Teil an der gespiegelten Position auftreten.
% Um dies zu implementieren benötigen wir nur ein einziges Nichtterminal (palindromcd).
% Dieses führt entweder zum leeren Wort, einem einzelnen "c", einem einzelnen "d" oder wieder zu sich selbst zurück, 
% umgeben von zwei c's oder zwei d's, jeweils eins vor und eins hinter dem Nichtterminal.
% Das Nichtterminal in der Mitte kann danach beliebig viele weitere Symbole um sich herum generieren.
% Dieser Vorgang wird dann gestoppt, wenn:
%   1) Das Nichtterminal "palindromcd" zu einem einzelnen Buchstaben führt, welcher dann in der Mitte des Wortes steht
% oder
%   2) Das Nichtterminal "palindromcd" zum leeren Wort führt wodurch die Mitte des Wortes leer bleibt und die Anzahl an Symbolen gerade ist.

palindromcd --> [].                                     % führt zur leeren Liste
palindromcd --> [c].                                    % ein c wird gelesen
palindromcd --> [d].                                    % ein d wird gelesen
palindromcd --> [c], palindromcd ,[c].                  % ein c wird gelesen, danach geht es zurück zum paindromcd, anschließend wird noch ein c gelesen
palindromcd --> [d], palindromcd, [d].                  % ein d wird gelesen, danach geht es zurück zum paindromcd, anschließend wird noch ein d gelesen


%
%% Testen (optional)
% Diese Tests kannst du manuell in deinem Prolog Interpreter durchführen.

% ?- palindromcd([c,c,d,c,c],[]).
% -> true.

% ?- palindromcd([d,d],[]).
% -> true.

% Anfrage, um alle möglichen Palindrome zu generieren:
% ?- palindromcd(X,[]). 
