%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                               %
% Übung: fsa_zu_dcg                                                             %
%                                                                               %
% Aufgabe 1)                                                                    %
% Geben Sie eine rechtslineare Grammatik an, die die Sprache generiert,         %
% die von folgendem endlichen Automaten akzeptiert wird:                        %
%                                                                               %
%                                                                               %
%                b            a                                                 %
%              ___     _______________                                          %
%             |   |   |               |                                         %
%             |___↓____            ___↓______                                   %            
%             |       |            ||------||  ←-----|                          %
%  start  ->  |  q0   |            ||  q1  ||        | b                        %
%             |_______|            ||______||  ______|                          %
%                    ↑             ----------                                   %
%                    |________________|                                         %               
%                             b                                                 %
%                                                                               %                         
% Eine übersichtlichere Abbildung des Automaten findest du auf                  %
% Seite 26 von Foliensatz 7.                                                    %
%                                                                               %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%
%% Erläuterung zu Aufgabe 1)
%
% Eine Grammatik ist rechtslinear, wenn jede ihrer Regeln die Form A → a B
% oder die Form A → a hat (a steht für ein Terminal und A und B für Nichtterminale).
% Zustand q0 wird in unserer Grammatik durch das Nichtterminal A, und Zustand q1 durch das Nichtterminal B dargestellt.
% Der Startzustand q0 des oben abgebildeten Automaten kann mit einem Loop eine beliebige Menge an b's erzeugen.
% Dies kann durch die Grammatikregel A → b, A dargestellt werden, welche in jedem Durchlauf ein b generiert 
% und wieder zu dem selben Zustand zurückkehrt.
% Dies geschieht so lange, bis der Übergang zu q1 benutzt wird, welcher ein einzelnes a erzeugt. Dieser Übergang ist zwingend
% notwendig um ein Wort zu erzeugen, da nur q1 ein Endzustand ist, aber nicht q0.
% Da unser Wort mit der Generierung eines einzelnen a's schon zuende sein kann da wir uns in einem Endzustand befinden,
% wird die Regel A → a definiert, welche kein weiteres Nichtterminal erzeugt und somit den Durchlauf beendet.
% In dem anderen Fall, in dem es nach der Generierung des a's noch weitergeht, wird ein a sowie das Nichtterminal B,
% welches Zustand q1 definiert, generiert. In unserer Grammatik wird diser Übergang von der Regel A → a, B repräsentiert.
% Nun befinden wir uns in Zustand q1 und haben wieder die Möglichkeit, mit einem Loop beliebig viele b's zu generieren.
% Dies wird durch die Regel B → b, B dargestellt. 
% Da wir durch diesen Loop immer wieder zu unserem Endzustand zurückkehren, kann das Wort an dieser Stelle zu jeder Zeit 
% beendet werden. Hierfür gibt es die Grammatikregel B → b.
% Die letzte Möglichkeit, die in Zustand q1 möglich ist, ist der Übergang zurück zu q0, welcher mit der Generierung eines a's einhergeht.
% Dies wird durch die Regel B → a, A dargestellt. Nun sind wie wieder im Startzustand und der Durchlauf beginnt von vorne.

%               
%%% Lösung Aufgabe 1)

%     A → a, B        % Übergang von q0 nach q1, Generierung von einem a
%     A → a           % Generierung von einem a 
%     A → b, A        % Übergang von q0 nach q0, Generierung von einem b
%     B → a, A        % Übergang von q1 nach q0, Generierung von einem a
%     B → b, B        % Übergang von q1 nach q1, Generierung von einem b
%     B → b           % Generierung von einem b


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                       %
% Aufgabe 2)                                            %                       
% Schreiben Sie die Grammatik als DCG in Prolog.        %           
%                                                       %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%
% DCG's können in Prolog einfach dargestellt werden.
% Nichtterminale werden kleingeschrieben und Terminale stehen kleingeschrieben in einer Liste.
% Die Übergänge werden durch einen Pfeil bestehend aus zwei Bindestrichen und dem größer-Zeichen dargestellt: -->
% Eine Grammatikregel der Form C → c sieht in Prolog-Notation also so aus: c --> [c].

%
%%% Lösung Aufgabe 2)

q0 --> [a], q1.                 % Übergang von q0 nach q1, Generierung von einem a                 
q0 --> [a].                     % Generierung von einem a
q0 --> [b], q0.                 % Übergang von q0 nach q0, Generierung von einem b
q1 --> [a], q0.                 % Übergang von q1 nach q0, Generierung von einem a
q1 --> [b], q1.                 % Übergang von q1 nach q1, Generierung von einem b
q1 --> [b].                     % Generierung von einem b


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                       %
% Aufgabe 3)                                            %
% Welche Sprache wird von der Grammatik generiert,      %
% welche Eigenschaften haben alle Wörter der Sprache?   %
%                                                       %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%
%% Erläuterung zu Aufgabe 3)
% Das Nichtterminal A unserer Grammatik kann mit Regel A → b, A , welche ein b generiert und wieder zu sich selbst zurück führt, 
% eine beliebige Menge an b's erzeugen. Dies wird in einem Regulären Ausdruck mit einem Kleene-Stern dargestellt: b*
% Das Generieren von b's geschieht so lange, bis Regel A → a oder A → a, B benutzt wird, welche ein einzelnes a generiert 
% und entweder den Durchlauf beendet oder durch Nichtterminal B noch weitere Symbole erzeugt.
% Die Generierung eines a's ist also zwingend notwendig um ein Wort zu erzeugen, da nur B ein Endzustand ist, aber nicht A.
% Auf die beliebige Erzeugung von b's in unserem Regulären Ausdruck muss nun also ein einzelnes a folgen, 
% sodass unser Zwischenstand so aussieht: b* a
% Wenn unser Wort nach der Generierung des a's noch weitergeht, können wieder beliebig viele b's durch die Regel B → b, B generiert werden.
% Um diesen Loop zu beenden, kann Regel B → b angewendet werden, welche noch ein b generiert und dann das Wort beendet.
% Um diese Möglichkeiten erweitert sieht der Reguläre Ausdruck folgendermaßen aus: b* a b* 
% Da aber auch Regel B → a, A angewendet werden kann, welche ein a generiert und zurück zu Nichtterminal A führt, 
% von wo aus unsere Symbolgenerierung von vorne beginnt, müssen wir unseren regulären Ausdruck so erweitern, dass
% unser bisheriger Ausdruck (b* a b*) beliebig oft, aber mindestens einmal durchgeführt werden kann.
% Dies wird mit einem Plus-Zeichen dargestellt: (b* a b*)+
% Da bei Benutzung der Regel B → a, A jedoch wieder ein einzelnes a generiert wird, um zurück zu Nichtterminal A zu kommen,
% muss der reguläre Ausdruck noch etwas abgeändert werden. Um von Nichtterminal A aus das Wort zu beenden muss nämlich wieder 
% ein einzelnes a generiert werden. Bei allen Worten der Sprache muss die Anzahl der a's demnach ungerade sein.
% Um dies darzustellen kennzeichnen wir das a im regulären Ausdruck mit einem hochgestellten n, welches wir danach als 
% größer-gleich 1 und ungerade definieren: (b* a^n b*)+ | n ≥ 1, n ist ungerade


%
%%% Lösung Aufgabe 3)

% Die gegebene Grammatik generiert die Sprache L = {(b* a^n b*)+ | n ≥ 1, n ist ungerade}.

% Alle Wörter der Sprache haben die Eigenschaft, dass mindestens ein a vorkommt und die Anzahl an a's immer ungerade ist.
% Bei den b's hingegen können an jeder Position des Wortes beliebig viele, oder auch garkeins auftauchen.
