%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                               %
% Unterschiedliche Grammatiken in Prolog und ihre Aufrufe       %
%                                                               %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Grammatik 1 ohne Listen, keine Rekursion möglich
s1(W1,W2,W3,W4,W5):- det1(W1), n1(W2), v1(W3), det1(W4), n1(W5).        % generiert einen Satz s mit 5 Wörtern aus W1 - W5
% Lexikon:  
det1(eine).                                                             % eine ist ein Determinierer
det1(die).                                                              % die ist ein Determinierer
det1(keine).                                                            % keine ist ein Determinierer
n1(maus).                                                               % maus ist ein Nomen
n1(katze).                                                              % katze ist ein Nomen
v1(jagt).                                                               % jagt ist ein Verb
v1(klaut).                                                              % klaut ist ein Verb

% Anfrage für "Eine Katze klaut eine Maus":
% ?- s1(eine, katze, klaut, eine, maus).
% Anfrage für alle von der Grammatik generierbaren Sätze:
% ?- s1(A, B, C, D, E).

% Grammatik 2 mit append/3 am Ende, sehr ineffizient:
s2(L3):- np2(L1), vp2(L2), append(L1,L2,L3).                            % generiert S bestehend aus einer NP und einer VP
vp2(L3):- v2(L1), np2(L2), append(L1,L2,L3).                            % generiert eine VP 
% Lexikon:
det2([eine]).                                                           % eine ist ein Determinierer
det2([die]).                                                            % die ist ein Determinierer
det2([keine]).                                                          % keine ist ein Determinierer
n2([maus]).                                                             % maus ist ein Nomen
n2([katze]).                                                            % katze ist ein Nomen
v2([jagt]).                                                             % jagt ist ein Verb
v2([klaut]).                                                            % klaut ist ein Verb

% Anfrage für "Eine Katze klaut eine Maus":
% ?- s2([eine, katze, klaut, eine, maus]).
% Anfrage für alle von der Grammatik generierbaren S�tze:
% ?- s2(X).

% Grammatik 3 append/3 am Anfang, ebenfalls ineffizient: 
s3(L3):- append(L1,L2,L3), np3(L1), vp3(L2).                            % generiert S bestehend aus einer NP und einer VP
np3(L3):- append(L1,L2,L3), det3(L1), n3(L2).                           % generiert eine NP aus einem Determinierer und einem Nomen
vp3(L3):- append(L1,L2,L3), v3(L1), np3(L2).                            % generiert eine VP aus einem Verb und einer NP

% Lexikon (wie bei Grammatik 2): 
det3([eine]).
det3([die]).
det3([keine]).
n3([maus]).
n3([katze]).
v3([jagt]).
v3([klaut]).

% Anfrage für "Eine Katze klaut eine Maus":
% ?- s3([eine, katze, klaut, eine, maus]).
% Anfrage für alle von der Grammatik generierbaren Sätze:
% ?- s3(X).

% Grammatik 4 mit Differenzlisten, Problem hierbei: durch die vielen Variablen
% verliert die Grammatik schnell an Lesbarkeit:

s4(A,C):- np4(A,B), vp4(B,C).                                           % generiert einen Satz aus einer NP und einer VP aus den 
                                                                        % jeweiligen Subsequenzen 
np4(A,C):- det4(A,B), n4(B,C).                                          % generiert eine NP aus einem Determinierer und einem Nomen
vp4(A,C):- v4(A,B), np4(B,C).                                           % generiert eine VP aus einem Verb und einer NP

% Lexikon:
det4([eine|A],A).
det4([die|A],A).
det4([keine|A],A).
n4([maus|A],A).
n4([katze|A],A).
v4([jagt|A],A).
v4([klaut|A],A).

% Anfrage für "Eine Katze klaut eine Maus":
% ?- s4([eine, katze, klaut, eine, maus],[]).
% Anfrage für alle von der Grammatik generierbaren Sätze:
% ?- s4(X,[]).

% Grammatik 5, gleiche Grammatik wie Grammatik 4, nur in einer 
% uebersichtlicheren Schreibweise
s5 --> np5, vp5.
np5 --> det5, n5.
vp5 --> v5, np5.

% Lexikon:
det5 --> [eine].
det5 --> [die].
det5 --> [keine].
n5 --> [maus].
n5 --> [katze].
v5 --> [jagt].
v5 --> [klaut].

% Anfrage für "Eine Katze klaut eine Maus":
% ?- s5([eine, katze, klaut, eine, maus],[]).
% Anfrage für alle von der Grammatik generierbaren Sätze:
% ?- s5(X,[]).
