%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                       %
% Übung: externe/interne Notation                                       %
% Übertragen Sie die folgenden DCGs in die interne Prolognotation:      %
%                                                                       %
%     1)   a --> b, c.                                                  %
%     2)   b --> c.                                                     %
%     3)   c --> [].                                                    %
%     4)   c --> [ha,hu].                                               %
%     5)   d --> a, b, c, d.                                            %
%     6)   d --> a, [hu], c.    % Zusatzaufgabe                         %
%                                                                       %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%
%% Erläuterung:
% Bei der DCG-Darstellung aus der Aufgabenstellung handelt es sich um “notational sugar” von Prolog.
% DCG-Klauseln werden intern umgewandelt in Klauseln mit Differenzlisten (siehe Foliensatz 7 Seite 9). 
% Eine Differenzliste ist ein Paar von Listen (L1,L2), wobei L2 ein Suffix von L1 repräsentiert.
% Die Elemente der Differenzliste sind die nach Abzug von Suffix L2 verbleibenden Elemente in L1.

% Um die Grammatikregel a --> b in der internen Notation darzustellen, muss also eine Differenzliste erstellt werden, 
% welche aus zwei Listen (A und B) besteht. Dies würde dann folgendermaßen aussehen:
%     a(A,B) :-
%            b(A,B).
% Differenzliste (A,B) enthält hierbei die Elemente aus A nach Abzug von Suffix B.

% Um die Grammatikregel a --> b, c darzustellen benötigen wir entsprechend des zusätzlichen Nichtterminals 
% noch ein zusätzliches Suffix, nämlich C. Liste B ist also Suffix von Liste A und Liste C Suffix von Liste B.
% Für jedes weitere Nichtterminal, zu dem eine Regel führt, wird auch ein weiteres Suffix benötigt.
% Die fertige interne Notation sieht dann so aus:
%     a(A,C) :-
%            b(A,B),
%            c(B,C).

% Die Regel a --> [a], welche nicht zu einem weiteren Nichtterminal sondern nur zu einem Terminal führt, 
% sieht in der internen Prolog-Notation so aus: 
%     a([a|A],A).
% Hierbei nimmt das Prädikat a wieder zwei Listen als Argumente, welche eine Differenzliste darstellen.
% Die erste Liste wird in Head und Tail aufgeteilt, wobei der Head das Terminal ist, welches unsere Regel generiert.
% Der Tail der ersten Liste (in unserem Fall repräsentiert von der Variable A) ist eine Variable, welche gleichzeitig 
% auch die zweite Liste ist. Die Elemente der Differenzliste ([a|A],A) bestehen also nur aus dem Head der ersten Liste,
% nämlich dem von unserer Grammatikregel generierten Terminal [a], da Liste 2 (A) von Liste 1 ([a|A]) abgezogen wird.

% Nun gibt es noch einen weiteren Fall, bei welchem eine Grammatikregel sowohl Nichtterminale als auch Terminale generiert.
% Die Regel a --> b, [a]. hat in interner Notation die folgende Form:
%     a(A, C) :-
%             b(A, B),
%             B=[a|C].
% Um den Übergang zum Nichtterminal b darzustellen wird Suffix B von Liste A benötigt, und ein weiteres Suffix C, 
% für die Darstellung des Terminals [a]. Anders als bei der Notation für eine Generierung eines einzelnen Terminals 
% gibt es hier keine eigene Differenzliste für das Terminal, sondern Liste B wird als Liste mit Terminal a als Head und Liste C als Tail definiert.
% Da in der Ankerregel Suffix C von Liste A abgezogen wird, bleibt am Ende Liste A ohne Suffix B und Liste B ohne Suffix C übrig.
% Liste A ohne Suffix B stellt Nichtterminal b und Liste B ohen Suffix C Terminal [a] dar.


%
%%% Lösungen:

% 1)
a --> b, c. 
% Die Regel führt zu zwei Nichtterminalen, weshalb zwei Suffixe (B und C) verwendet werden.

% a(A,C) :-
%        b(A,B),
%        c(B,C).

% 2)
b --> c.
% Die Regel führt zu einem Nichtterminal, weshalb nur ein Suffix (B) verwendet wird.

% b(A,B) :-
%        c(A,B).

% 3)
c --> [].
% Die Regel führt zu dem leeren Wort, weshalb eine Differenzliste, welche keine Elemente enthält, implementiert wird. 
% Liste A nach Abzug von Liste A ist leer.

% c(A,A).

% 4)
c --> [ha,hu].
% Die Regel führt zu einem Terminal, weshalb eine einzelne Differenzliste implementiert wird, bei der dieses Terminal übrug bleibt.

% c([ha, hu |A], A).

% 5)
d --> a, b, c, d.
% Die Regel führt zu vier Nichtterminalen, weshalb vier Suffixe von Liste A (B, C, D, E) in den Differenzlisten verwendet werden.

% d(A,E) :-
%        a(A,B),
%        b(B,C),
%        c(C,D),
%        d(D,E).

% 6)
d --> a, [hu], c.
% Die Regel führt zu einem Nichtterminal gefolgt von einem Terminal und wieder einem Nichtterminal.
% Daher werden in der Notation drei Suffixe von A verwendet, zwei für die beiden Nichtterminale uns eins für das Terminal.

% d(A,D) :-
%        a(A,B),
%        B=[hu|C],
%        c(C,D).


%
%% Testen (optional)
% Diese Tests kannst du manuell in deinem Prolog Interpreter durchführen.
% Mit listing/1 koennt ihr euch alle Regeln zu einem Praedikat in der internen Schreibweise ausgeben lassen.

% ?- listing(a).
% ?- listing(b).
% ?- listing(c).
% ?- listing(d).
